@@ -10,28 +10,27 @@
 *
 * $Project: Convert-Binary-C $
 * $Author: mhx $
-* $Date: 2002/04/15 23:26:46 +0200 $
-* $Revision: 1 $
-* $Snapshot: /Convert-Binary-C/0.06 $
+* $Date: 2003/01/01 12:29:55 +0100 $
+* $Revision: 4 $
+* $Snapshot: /Convert-Binary-C/0.07 $
 * $Source: /ctlib/pragma.h $
 *
 ********************************************************************************
 *
-* Copyright (c) 2002 Marcus Holland-Moritz. All rights reserved.
-* This program is free software; you can redistribute it and/or
-* modify it under the same terms as Perl itself.
+* Copyright (c) 2002-2003 Marcus Holland-Moritz. All rights reserved.
+* This program is free software; you can redistribute it and/or modify
+* it under the same terms as Perl itself.
 *
 *******************************************************************************/
 
-#ifndef _PRAGMA_H
-#define _PRAGMA_H
+#ifndef _CTLIB_PRAGMA_H
+#define _CTLIB_PRAGMA_H
 
 /*===== GLOBAL INCLUDES ======================================================*/
 
 /*===== LOCAL INCLUDES =======================================================*/
 
 #include "util/list.h"
-#include "ucpp/cpp.h"
 
 
 /*===== DEFINES ==============================================================*/
