@@ -10,16 +10,16 @@
 #
 # $Project: Convert-Binary-C $
 # $Author: mhx $
-# $Date: 2002/12/11 15:01:43 +0100 $
-# $Revision: 1 $
-# $Snapshot: /Convert-Binary-C/0.06 $
+# $Date: 2003/01/01 12:29:55 +0100 $
+# $Revision: 3 $
+# $Snapshot: /Convert-Binary-C/0.07 $
 # $Source: /ctlib/t_config.pl $
 #
 ################################################################################
 # 
-# Copyright (c) 2002 Marcus Holland-Moritz. All rights reserved.
-# This program is free software; you can redistribute it and/or
-# modify it under the same terms as Perl itself.
+# Copyright (c) 2002-2003 Marcus Holland-Moritz. All rights reserved.
+# This program is free software; you can redistribute it and/or modify
+# it under the same terms as Perl itself.
 # 
 ################################################################################
 
@@ -28,12 +28,7 @@ use Tokenizer;
 
 $t = new Tokenizer tokfnc => \&tok_code, tokstr => 'option';
 
-@C99 = qw(
-  HasCPPComments
-  HasMacroVAARGS
-);
-
-$t->addtokens( '', qw(
+@OPT = qw(
   UnsignedChars
   Warnings
   PointerSize
@@ -50,16 +45,40 @@ $t->addtokens( '', qw(
   Define
   Assert
   DisabledKeywords
+  KeywordMap
   ByteOrder
   EnumType
-));
+);
 
-# options only with ANSI C99
+@C99 = qw(
+  HasCPPComments
+  HasMacroVAARGS
+);
 
+$t->addtokens( '', @OPT );
 $t->addtokens( 'ANSIC99_EXTENSIONS', @C99 );
 
+$enums     = join "\n", map "  OPTION_$_,", @OPT;
+$enums_c99 = join "\n", map "  OPTION_$_,", @C99;
+$switch    = $t->makeswitch;
+
 open OUT, ">$ARGV[0]" or die $!;
-print OUT $t->makeswitch;
+print OUT <<END;
+typedef enum {
+$enums
+#ifdef ANSIC99_EXTENSIONS
+$enums_c99
+#endif
+  INVALID_OPTION
+} ConfigOption;
+
+ConfigOption GetConfigOption( const char *option )
+{
+$switch
+unknown:
+  return INVALID_OPTION;
+}
+END
 close OUT;
 
 sub tok_code {
