@@ -10,16 +10,16 @@
 *
 * $Project: Convert-Binary-C $
 * $Author: mhx $
-* $Date: 2002/04/15 23:26:49 +0200 $
-* $Revision: 1 $
-* $Snapshot: /Convert-Binary-C/0.06 $
+* $Date: 2003/01/07 22:26:25 +0100 $
+* $Revision: 4 $
+* $Snapshot: /Convert-Binary-C/0.07 $
 * $Source: /ctlib/ucpp/config.h $
 *
 ********************************************************************************
 *
-* Copyright (c) 2002 Marcus Holland-Moritz. All rights reserved.
-* This program is free software; you can redistribute it and/or
-* modify it under the same terms as Perl itself.
+* Copyright (c) 2002-2003 Marcus Holland-Moritz. All rights reserved.
+* This program is free software; you can redistribute it and/or modify
+* it under the same terms as Perl itself.
 *
 *******************************************************************************/
 
@@ -52,10 +52,21 @@
 #define NATIVE_UINTMAX u_64
 #define NATIVE_INTMAX  i_64
 
+#define NATIVE_SIGNED           i_64
+#define NATIVE_UNSIGNED         u_64
+#define NATIVE_UNSIGNED_BITS    64
+#define NATIVE_SIGNED_MIN       (-9223372036854775807LL - 1)
+#define NATIVE_SIGNED_MAX       9223372036854775807LL
+
 #else
 
 #define SIMUL_UINTMAX
 
+#undef NATIVE_SIGNED
+#define SIMUL_ARITH_SUBTYPE     u_32
+#define SIMUL_SUBTYPE_BITS      32
+#define SIMUL_NUMBITS           64
+
 #endif
 
 /*----------------------------------*/
@@ -75,6 +86,10 @@
 /* other stuff */
 /*-------------*/
 
+#define ARITHMETIC_CHECKS
+
+/* #define LOW_MEM */
+
 #define NO_UCPP_ERROR_FUNCTIONS
 
 #define MAX_CHAR_VAL 256
